@@ -5,8 +5,16 @@ extern "C" {
 #ifndef MAXMINDDB_H
 #define MAXMINDDB_H
 
+/* Request POSIX.1-2008. However, we want to remain compatible with
+ * POSIX.1-2001 (since we have been historically and see no reason to drop
+ * compatibility). By requesting POSIX.1-2008, we can conditionally use
+ * features provided by that standard if the implementation provides it. We can
+ * check for what the implementation provides by checking the _POSIX_VERSION
+ * macro after including unistd.h. If a feature is in POSIX.1-2008 but not
+ * POSIX.1-2001, check that macro before using the feature (or check for the
+ * feature directly if possible). */
 #ifndef _POSIX_C_SOURCE
-#define _POSIX_C_SOURCE 200112L
+#define _POSIX_C_SOURCE 200809L
 #endif
 
 #include "maxminddb_config.h"
@@ -20,7 +28,7 @@ extern "C" {
 #include <winsock2.h>
 #include <ws2tcpip.h>
 /* libmaxminddb package version from configure */
-#define PACKAGE_VERSION "1.3.1"
+#define PACKAGE_VERSION "1.6.0"
 
 typedef ADDRESS_FAMILY sa_family_t;
 
@@ -79,7 +87,7 @@ typedef ADDRESS_FAMILY sa_family_t;
 
 #if !(MMDB_UINT128_IS_BYTE_ARRAY)
 #if MMDB_UINT128_USING_MODE
-typedef unsigned int mmdb_uint128_t __attribute__ ((__mode__(TI)));
+typedef unsigned int mmdb_uint128_t __attribute__((__mode__(TI)));
 #else
 typedef unsigned __int128 mmdb_uint128_t;
 #endif
@@ -87,7 +95,7 @@ typedef unsigned __int128 mmdb_uint128_t;
 
 /* This is a pointer into the data section for a given IP address lookup */
 typedef struct MMDB_entry_s {
-    struct MMDB_s *mmdb;
+    const struct MMDB_s *mmdb;
     uint32_t offset;
 } MMDB_entry_s;
 
@@ -131,10 +139,12 @@ typedef struct MMDB_entry_data_s {
     uint32_t type;
 } MMDB_entry_data_s;
 
-/* This is the return type when someone asks for all the entry data in a map or array */
+/* This is the return type when someone asks for all the entry data in a map or
+ * array */
 typedef struct MMDB_entry_data_list_s {
     MMDB_entry_data_s entry_data;
     struct MMDB_entry_data_list_s *next;
+    void *pool;
 } MMDB_entry_data_list_s;
 
 typedef struct MMDB_description_s {
@@ -142,6 +152,10 @@ typedef struct MMDB_description_s {
     const char *description;
 } MMDB_description_s;
 
+/* WARNING: do not add new fields to this struct without bumping the SONAME.
+ * The struct is allocated by the users of this library and increasing the
+ * size will cause existing users to allocate too little space when the shared
+ * library is upgraded */
 typedef struct MMDB_metadata_s {
     uint32_t node_count;
     uint16_t record_size;
@@ -158,13 +172,23 @@ typedef struct MMDB_metadata_s {
         size_t count;
         MMDB_description_s **descriptions;
     } description;
+    /* See above warning before adding fields */
 } MMDB_metadata_s;
 
+/* WARNING: do not add new fields to this struct without bumping the SONAME.
+ * The struct is allocated by the users of this library and increasing the
+ * size will cause existing users to allocate too little space when the shared
+ * library is upgraded */
 typedef struct MMDB_ipv4_start_node_s {
     uint16_t netmask;
     uint32_t node_value;
+    /* See above warning before adding fields */
 } MMDB_ipv4_start_node_s;
 
+/* WARNING: do not add new fields to this struct without bumping the SONAME.
+ * The struct is allocated by the users of this library and increasing the
+ * size will cause existing users to allocate too little space when the shared
+ * library is upgraded */
 typedef struct MMDB_s {
     uint32_t flags;
     const char *filename;
@@ -178,6 +202,7 @@ typedef struct MMDB_s {
     uint16_t depth;
     MMDB_ipv4_start_node_s ipv4_start_node;
     MMDB_metadata_s metadata;
+    /* See above warning before adding fields */
 } MMDB_s;
 
 typedef struct MMDB_search_node_s {
@@ -189,43 +214,44 @@ typedef struct MMDB_search_node_s {
     MMDB_entry_s right_record_entry;
 } MMDB_search_node_s;
 
-    /* *INDENT-OFF* */
-    /* --prototypes automatically generated by dev-bin/regen-prototypes.pl - don't remove this comment */
-    extern int MMDB_open(const char *const filename, uint32_t flags, MMDB_s *const mmdb);
-    extern MMDB_lookup_result_s MMDB_lookup_string(MMDB_s *const mmdb,
-                                                   const char *const ipstr,
-                                                   int *const gai_error,
-                                                   int *const mmdb_error);
-    extern MMDB_lookup_result_s MMDB_lookup_sockaddr(
-               MMDB_s *const mmdb,
-               const struct sockaddr *const sockaddr,
-               int *const mmdb_error);
-    extern int MMDB_read_node(MMDB_s *const mmdb, uint32_t node_number,
-                              MMDB_search_node_s *const node);
-    extern int MMDB_get_value(MMDB_entry_s *const start,
-                              MMDB_entry_data_s *const entry_data,
-                              ...);
-    extern int MMDB_vget_value(MMDB_entry_s *const start,
-                               MMDB_entry_data_s *const entry_data,
-                               va_list va_path);
-    extern int MMDB_aget_value(MMDB_entry_s *const start,
-                               MMDB_entry_data_s *const entry_data,
-                               const char *const *const path);
-    extern int MMDB_get_metadata_as_entry_data_list(
-               MMDB_s *const mmdb, MMDB_entry_data_list_s **const entry_data_list);
-    extern int MMDB_get_entry_data_list(
-               MMDB_entry_s *start, MMDB_entry_data_list_s **const entry_data_list);
-    extern void MMDB_free_entry_data_list(MMDB_entry_data_list_s *const entry_data_list);
-    extern void MMDB_close(MMDB_s *const mmdb);
-    extern const char *MMDB_lib_version(void);
-    extern int MMDB_dump_entry_data_list(FILE *const stream,
-                                         MMDB_entry_data_list_s *const entry_data_list,
-                                         int indent);
-    extern const char *MMDB_strerror(int error_code);
-    /* --prototypes end - don't remove this comment-- */
-    /* *INDENT-ON* */
+extern int
+MMDB_open(const char *const filename, uint32_t flags, MMDB_s *const mmdb);
+extern MMDB_lookup_result_s MMDB_lookup_string(const MMDB_s *const mmdb,
+                                               const char *const ipstr,
+                                               int *const gai_error,
+                                               int *const mmdb_error);
+extern MMDB_lookup_result_s
+MMDB_lookup_sockaddr(const MMDB_s *const mmdb,
+                     const struct sockaddr *const sockaddr,
+                     int *const mmdb_error);
+extern int MMDB_read_node(const MMDB_s *const mmdb,
+                          uint32_t node_number,
+                          MMDB_search_node_s *const node);
+extern int MMDB_get_value(MMDB_entry_s *const start,
+                          MMDB_entry_data_s *const entry_data,
+                          ...);
+extern int MMDB_vget_value(MMDB_entry_s *const start,
+                           MMDB_entry_data_s *const entry_data,
+                           va_list va_path);
+extern int MMDB_aget_value(MMDB_entry_s *const start,
+                           MMDB_entry_data_s *const entry_data,
+                           const char *const *const path);
+extern int MMDB_get_metadata_as_entry_data_list(
+    const MMDB_s *const mmdb, MMDB_entry_data_list_s **const entry_data_list);
+extern int
+MMDB_get_entry_data_list(MMDB_entry_s *start,
+                         MMDB_entry_data_list_s **const entry_data_list);
+extern void
+MMDB_free_entry_data_list(MMDB_entry_data_list_s *const entry_data_list);
+extern void MMDB_close(MMDB_s *const mmdb);
+extern const char *MMDB_lib_version(void);
+extern int
+MMDB_dump_entry_data_list(FILE *const stream,
+                          MMDB_entry_data_list_s *const entry_data_list,
+                          int indent);
+extern const char *MMDB_strerror(int error_code);
 
-#endif                          /* MAXMINDDB_H */
+#endif /* MAXMINDDB_H */
 
 #ifdef __cplusplus
 }
